<?php
	/**
	 *	eBag
	 *
	 *	Open source company administration manager
	 *
	 *	@package	eBag
	 *	@author		Eper Kalman
	 *	@copyright	Copyright (c) 2012-2015, Eper Kalman
	 *	@license	GNU Affero General Public License Version 3 (http://www.gnu.org/licenses/agpl-3.0.html)
	 *	@since		Version 6.0.0
	 */

	/*
	 * Interleaved 2 of 5 bar code generator
	 */
	 
	if (! defined('eBag'))	{
	
	      include('../views/logout.php'); 
	      
	      exit();
	}

	class INTERLEAVED_2_OF_5_BAR_CODE	{

		var $CODE;

		var $PRINT_CODE_STRING	=	FALSE;

		var $BAR_DEFINITION	=	'';

		var $SCALE_FACTOR	=	1;

		var $NARROW_WIDTH_RATIO	=	2.5;

		var $NARROW_BAR_WIDTH	=	2;

		var $BAR_CODE_HEIGHT	=	0;

		var $PADDING		=	10;

		var $FONT_SIZE		=	10;

		var $FONT_FILE		=	FALSE;

		var $CHECK_DIGIT	=	FALSE;

		var $INTERLEAVED_2_OF_5_DEFINITIONS	=	array	(	'0'	=>	'NNWWN',
										'1'	=>	'WNNNW',
										'2'	=>	'NWNNW',
										'3'	=>	'WWNNN',
										'4'	=>	'NNWNW',
										'5'	=>	'WNWNN',
										'6'	=> 	'NWWNN',
										'7'	=> 	'NNNWW',
										'8'	=>	'WNNWN',
										'9'	=>	'NWNWN'		);

		function __construct()	{
		
			if (defined('BAR_FONT_FILE')) $this->FONT_FILE = BAR_FONT_FILE;
		}
										
		public function code($string = FALSE)	{

			if ($string) $this->CODE = $this->CODE_ORIGIN = $string;
		}

		public function scale($factor = FALSE)	{

			if ($factor > 1) $this->SCALE_FACTOR = $factor;
		}

		public function width($pixel = FALSE)	{

			if ((integer)$pixel > 0) $this->NARROW_BAR_WIDTH = (integer)$pixel;
		}

		public function height($height = FALSE)	{

			if ($height > 0) $this->BAR_CODE_HEIGHT = $height;
		}

		public function padding($padding = FALSE)	{

			if ($padding > 0) $this->PADDING = $padding;
		}

		public function print_code()	{

			if ($this->FONT_FILE) $this->PRINT_CODE_STRING = TRUE;
		}

		public function font_size($size)	{

			if ($size > 5) $this->FONT_SIZE = $size;
		}

		public function generate()	{

			if (! $this->CODE) return FALSE;

			# generate definition

			if (! $this->generate_bar_definition()) return FALSE;

			# set the scale factor

			$this->NARROW_BAR_WIDTH = (integer)($this->NARROW_BAR_WIDTH * $this->SCALE_FACTOR);

			# calculate width

			$this->calculate_bar_dimensions();

			# draw the bar code

			$this->draw_bar();

			return TRUE;
		}

		private function generate_bar_definition()	{

			# interleaved 2 of 5 supports only odd number of digits

			$length	=	strlen($this->CODE);

			if ($length % 2)	{

				$length++;

				$this->CODE = $this->CODE_ORIGIN = '0'.$this->CODE;
			}

			# calculate checksum digit

			if ($this->CHECK_DIGIT)	{

				$even_sum = $odd_sum = 0;

				for ($i=0; $i<$length; $i++)	{

					$digit	= substr($this->CODE, $i, 1);

					if (! isset($this->INTERLEAVED_2_OF_5_DEFINITIONS[$digit]))	{

						$error = TRUE;

						break;
					}

					if ($digit % 2) {$even_sum += $digit;} else {$odd_sum += $digit;}
				}

				if (isset($error)) return FALSE;

				$odd_sum = 3 * $odd_sum;

				$check_digit = ($odd_sum + $even_sum) % 10;

				if ($check_digit != 0) $check_digit = 10-$check_digit;

				$this->CODE = $this->CODE.'0'.$check_digit;

				$length += 2;
			}

			# generate definition

			for ($i=0; $i<$length; $i=$i+2)	{

				$BAR_char	=	substr($this->CODE, $i, 1);
				$SPACE_char	=	substr($this->CODE, $i+1, 1);

				if ((! isset($this->INTERLEAVED_2_OF_5_DEFINITIONS[$BAR_char])) or (! isset($this->INTERLEAVED_2_OF_5_DEFINITIONS[$SPACE_char])))	{

					$error = TRUE;

					break;
				}

				# mix the definitions

				for ($j=0; $j<5; $j++)	{

					$B_DEF	=	substr($this->INTERLEAVED_2_OF_5_DEFINITIONS[$BAR_char], $j, 1);

					if ($B_DEF == 'W') {$B_DEF='B';} else {$B_DEF='b';}

					$S_DEF	=	substr($this->INTERLEAVED_2_OF_5_DEFINITIONS[$SPACE_char], $j, 1);

					if ($S_DEF == 'N') $S_DEF='w';

					$this->BAR_DEFINITION .= ($B_DEF.$S_DEF);
				}
			}

			if (isset($error)) return FALSE;

			# adding start and end

			$quitet_zone = 'wwwwwwwwww';

			$this->BAR_DEFINITION = $quitet_zone.'bwbw'.$this->BAR_DEFINITION.'Bwb'.$quitet_zone;

			return TRUE;
		}

		private function calculate_bar_dimensions()	{

			$this->LENGTH	= strlen($this->BAR_DEFINITION);

			$this->BAR_CODE_WIDTH	=	0;

			$this->WIDE_BAR_WIDTH = (integer)($this->NARROW_BAR_WIDTH * $this->NARROW_WIDTH_RATIO);

			for ($i=0; $i<$this->LENGTH; $i++)	{

				$def	=	substr($this->BAR_DEFINITION, $i, 1);

				if (($def=='B') or ($def=='W')) {

					$this->BAR_CODE_WIDTH += $this->WIDE_BAR_WIDTH;

				}	else	  {

					$this->BAR_CODE_WIDTH += $this->NARROW_BAR_WIDTH;
				}
			}

			if (! $this->BAR_CODE_HEIGHT)  $this->BAR_CODE_HEIGHT = (integer)($this->BAR_CODE_WIDTH * 15/100);
		}

		private function draw_bar()	{

			# Calculate code string coords

			if ($this->PRINT_CODE_STRING)	{

				$box   = imagettfbbox($this->FONT_SIZE, 0, $this->FONT_FILE, $this->CODE_ORIGIN);

				$min_x = min( array($box[0], $box[2], $box[4], $box[6]) );
				$max_x = max( array($box[0], $box[2], $box[4], $box[6]) );
				$min_y = min( array($box[1], $box[3], $box[5], $box[7]) );
				$max_y = max( array($box[1], $box[3], $box[5], $box[7]) );

				$code_string_width  = ( $max_x - $min_x );
				$code_string_height = ( $max_y - $min_y ) + 5; 

			}	else	  {

				$code_string_height = 0; 
			}

			# Create image

			$image_width	=	$this->BAR_CODE_WIDTH + 2 * $this->PADDING;

			$image_height	=	$this->BAR_CODE_HEIGHT + 2 * $this->PADDING + $code_string_height;

			$image = imagecreate($image_width, $image_height);

			$white = imagecolorallocate($image, 255, 255, 255);

			imagefill($image, 0, 0, $white);

			# Draw the bars

			$black = imagecolorallocate($image, 0, 0, 0);

			$X_COOR = $this->PADDING;

			for ($i=0; $i<$this->LENGTH; $i++)	{

				$def	=	substr($this->BAR_DEFINITION, $i, 1);

				switch ($def)	{

					case('b')	:	imagefilledrectangle($image, $X_COOR, $this->PADDING, $X_COOR + $this->NARROW_BAR_WIDTH - 1, $this->BAR_CODE_HEIGHT+$this->PADDING, $black);

								$X_COOR += $this->NARROW_BAR_WIDTH;

								break;

					case('B')	:	imagefilledrectangle($image, $X_COOR, $this->PADDING, $X_COOR + $this->WIDE_BAR_WIDTH - 1, $this->BAR_CODE_HEIGHT + $this->PADDING, $black);

								$X_COOR += $this->WIDE_BAR_WIDTH;

								break;

					case('w')	:	$X_COOR += $this->NARROW_BAR_WIDTH;

								break;

					case('W')	:	$X_COOR += $this->WIDE_BAR_WIDTH;

								break;
				}
			}

			# Print code

			if ($this->PRINT_CODE_STRING)	imagettftext( $image, $this->FONT_SIZE, 0, (integer)(($image_width-$code_string_width)/2), $this->BAR_CODE_HEIGHT + $this->PADDING + $code_string_height, $black, $this->FONT_FILE, $this->CODE_ORIGIN);

			# Output

			header('Pragma: no-cache');
			header('Cache-Control: no-cache, no-store, must-revalidate');
			header('Expires: Sun, 10 Jan 2010 12:00:00 GMT');
			header('Last-Modified: '.@gmdate('D, d M Y H:i:s').' GMT');
			header('Content-type: image/png');

			imagepng($image);

			imagedestroy($image); 
		}
	}
?>
