<?php
	/**
	 *	eBag
	 *
	 *	Open source company administration manager
	 *
	 *	@package	eBag
	 *	@author		Eper Kalman
	 *	@copyright	Copyright (c) 2012-2015, Eper Kalman
	 *	@license	GNU Affero General Public License Version 3 (http://www.gnu.org/licenses/agpl-3.0.html)
	 *	@since		Version 6.0.0
	 */

	/**
	 *	eBag INIloader
	 *	Loads configuration file into mem once for all users
	 */

	/**
	 *	Usage:	$conf=INI_load(file, $reload);
	 *		echo $conf->get('key');
	 */

	class INI_load	{

		var $CONF = array();

		function __construct($file, $reload = FALSE)	{
		
			if (function_exists('apc_fetch')) {$apc = TRUE;} else {$apc = FALSE;}
		
			if ($apc)	{
			
				if  (apc_exists('eBagConfiguration') && (! $reload))	{
			
					$this->CONF = apc_fetch('eBagConfiguration');
					
					$this->define_constants();
		
					return;
				}
			}

			# parse (wait to reader lock if the configuration is just saved)

			if ($fh	= fopen($file, 'r'))	{
			
				$i = 0;
				
				while (! flock($fh, LOCK_SH))	{

					$i++; usleep(500);

					if ($i > 100000)	{

						fclose($fh); 
						
						exit;
					}
				}
				
				$this->CONF = @parse_ini_file($file, true);

				flock($fh, LOCK_UN);
				
				fclose($fh);
			}

			if ((! $this->CONF) or (! count($this->CONF))) exit('eBag fatal configuration error.  Please contact your system administrator !');
			
			# extend configuration with SQL date format
			
			$functions_path = dirname(dirname(__FILE__)).'/functions';
			
			if (! function_exists('sql_format')) include($functions_path.'/sql_format.php');
				
			$this->CONF['date']['sql_format'] = sql_format($this->CONF['date']['dateformat']);
			
			# decode entitites, important
			
			if (isset($this->CONF['payments']['currency']))	$this->CONF['payments']['currency'] =  html_entity_decode($this->CONF['payments']['currency'], ENT_COMPAT, 'UTF-8');
			
			if (isset($this->CONF['payments']['valuta'])) $this->CONF['payments']['valuta'] = html_entity_decode($this->CONF['payments']['valuta'], ENT_COMPAT, 'UTF-8');
			
			# define htmlspecicalchars flag too
			
			if (! function_exists('specialchars_flag')) include($functions_path.'/specialchars_flag.php');
			
			$this->CONF['specialchars']['flag'] = specialchars_flag();

			# store 
			
			if (isset($this->CONF['cache']['use_apc_cache']) && ($this->CONF['cache']['use_apc_cache']) && $apc) apc_store('eBagConfiguration', (array)$this->CONF, 0);
			
			$this->define_constants();
		}

		# Returns conf value by key

		public function get($section, $key)	{
		
			$val = FALSE;
			
			if (isset($this->CONF[$section][$key])) $val = $this->CONF[$section][$key];

			return $val;
		}
		
		# Extends configuration
		
		public function set($section, $key, $val)	{
		
		      $this->CONF[$section][$key] = $val;
			
		      if (isset($this->CONF['cache']['use_apc_cache']) && ($this->CONF['cache']['use_apc_cache']) && function_exists('apc_fetch')) apc_store('eBagConfiguration', (array)$this->CONF, 0);
		}
		
		# Define important constans
		
		private function define_constants()	{
		
			if (! defined('VERSION_NUMBER')) define('VERSION_NUMBER', $this->get('version', 'version_number'));
			
			if (! ($direction = $this->get('language', 'direction'))) $direction = 'ltr';
			
			if (! defined('TEXT_DIRECTION')) define('TEXT_DIRECTION', $direction);
			
			# set timezone

			ini_set('date.timezone', $this->get('date', 'timezone'));
		}
	}

?>
