<?php
	/**
	 *	eBag
	 *
	 *	Open source company administration manager
	 *
	 *	@package	eBag
	 *	@author		Eper Kalman
	 *	@copyright	Copyright (c) 2012-2015, Eper Kalman
	 *	@license	GNU Affero General Public License Version 3 (http://www.gnu.org/licenses/agpl-3.0.html)
	 *	@since		Version 6.0.0
	 */

	/**
	 * eBag sessionhandler
	 */

	class MysqlSession	{

		var $_base			=	'ebagtemp';

		var $_sessions			=	'sessions';

		var $_loggeds			=	'loggeds';
		
		var $_norestart			=	'norestart';

		var $_expire			=	600;

		var $_mysqli			=	FALSE;

		var $_active			=	FALSE;

		var $_loggedNow			=	FALSE;
		
		var $_readonly			=	FALSE;

		/**
		 * Sets vars for connection
		 */
		 
		function __construct()	{
		
			global $__CONF;
			
			$this->_host		=	(string)$__CONF->get('connection', 'host');
			
			$this->_user		=	(string)$__CONF->get('connection', 'user');
			
			$this->_password	=	(string)$__CONF->get('connection', 'password');
		}

		# sets the base

		public function base($base)	{

			$this->_base = $base;
		}
		
		# readonly session (like session_abort())
		
		public function readonly()	{
		
			$this->_readonly = TRUE;
		}

		# User login
		# Note: functions login & is_logged espects username properly escaped

		public function login($user=FALSE)	{

			if ((! $this->_active) or (! strlen($user)))	return FALSE;

			$SID = session_id();

			if ($SID == '') exit('No session started, can\'t set user to logged !');

			# First run the gc and delete expired sessions

			$this->session_gc();

			# check is logged to not log twice

			if ($this->is_logged($user)) return FALSE;

			$this->_mysqli->query("	INSERT INTO $this->_base.$this->_loggeds 
						VALUES 
						('$SID', '$user')	");

			$this->_loggedNow = TRUE;

			return TRUE;
		}

		# User logout

		public function logout($user=FALSE)	{

			$SID = $this->is_logged($user);

			if ($SID = session_id()) $this->delete_logged($SID);

			$this->_loggedNow = FALSE;
		}

		# Checks the user is logged returns SID

		public function is_logged($user=FALSE)	{

			if ((! $this->_active) or (! strlen($user)))	return FALSE;

			if ($this->_loggedNow) return session_id();

			$SID		=	FALSE;

			$time		=	time();

			if ($this->_mysqli)	{

				$result = $this->_mysqli->query("	SELECT `SID` 
									FROM $this->_base.$this->_loggeds 
									WHERE `user`='$user' LIMIT 1	");
				if ($result->num_rows)	{

					$row = $result->fetch_row();

					$SID = $row[0];
				}
			}

			if (($SID) && ($this->_expire))	{

				$result = $this->_mysqli->query("	SELECT `expiration` 
									FROM $this->_base.$this->_sessions 
									WHERE `SID`='$SID' AND expiration > $time LIMIT 1	");

				if (! $result->num_rows) $SID = FALSE;

			}

			return $SID;
		}

		# If the system was restarted delete the logged users & sessions

		private function empty_base()	{

			# delete the logged users

			$this->_mysqli->query("	DELETE 
						FROM $this->_base.$this->_loggeds	");

			$this->_mysqli->query("	DELETE 
						FROM $this->_base.$this->_sessions	");
		}

		# Opens new mysqli connection with utf-8 collation
		# Note: persisten connection is removed because of locks & transactions

		public function session_open()	{

			if (! $this->_mysqli)	{

				$this->_mysqli = new mysqli($this->_host, $this->_user, $this->_password, $this->_base);

				if ($this->_mysqli->connect_error) exit('Can\'t connect to mysql server ('.$this->_mysqli->connect_errno. ') '. $this->_mysqli->connect_error.' !');

				$this->_mysqli->set_charset("utf8");
				
				$this->_mysqli->query("	SET 
							NAMES 'utf8' 
							COLLATE 'utf8_bin'	");
			}

			# Check for system restart

			$result = $this->_mysqli->query("	SELECT `true` 
								FROM $this->_base.$this->_norestart 
								WHERE `true`='1' LIMIT 1	");

			if (! $result->num_rows)	{

				$this->empty_base();

				$this->_mysqli->query("	INSERT 
							INTO $this->_base.$this->_norestart
							VALUES ('1')	");
			}
		
			# run the garbage collector with 20% chance 

			if ((integer)rand(1,5) == 2) $this->session_gc();

			return TRUE;
		}

		# Write the values to memory or base

		public function session_write($SID, $value)	{
		
			if ($this->_readonly) return TRUE;

			$time		=	time();
			$expiration 	= 	$time + $this->_expire;

			# write to base

			$value		=	$this->_mysqli->real_escape_string($value);

			$result = $this->_mysqli->query("	INSERT INTO $this->_base.$this->_sessions 
								VALUES 
								('$SID', '$expiration', '$value')	");
								
			if ($result===FALSE) $this->_mysqli->query("	UPDATE $this->_base.$this->_sessions 
									SET `expiration`='$expiration', `value`='$value' 
									WHERE `SID`='$SID' LIMIT 1	");
			return TRUE;
		}

		# Read session variable

		public function session_read($SID)	{

			$value	=	'';
			$time	=	time();

			$result=$this->_mysqli->query("	SELECT `value` 
							FROM $this->_base.$this->_sessions 
							WHERE `SID`='$SID' AND expiration > $time LIMIT 1	");

			if ($result->num_rows)	{

				$row	=	$result->fetch_row();
				$value	=	$row[0];
			}

			return $value;
		}

		# Session close

		public function session_close()	{

			return TRUE;
		}

		# Destroy the session delete the values

		public function session_destroy($SID)	{

			$this->_mysqli->query("	DELETE 
						FROM $this->_base.$this->_sessions 
						WHERE `SID`='$SID' LIMIT 1	");


			$this->delete_logged($SID);

			return TRUE;
		}

		# The real garbage collector which
		# runs before data read

		public function session_gc()	{

			$time		=	time();

			$result=$this->_mysqli->query("	SELECT `SID` 
							FROM $this->_base.$this->_sessions 
							WHERE expiration < $time	");

			if ($result->num_rows)	{

				while ($row = $result->fetch_row())	{

					$SID=$row[0];

					$this->_mysqli->query("	DELETE 
								FROM $this->_base.$this->_sessions 
								WHERE `SID` = '$SID' LIMIT 1	");

					$this->delete_logged($SID);
				}
			}
		}

		# Garbage collector for set_save_handler
		# does nothing, only for syntax is here

		public function session_gc_fake ($lifetime)	{
			return;
		}

		# Delete logged user

		private function delete_logged($SID)	{

			if ($this->_mysqli) $this->_mysqli->query("	DELETE
									FROM $this->_base.$this->_loggeds
									WHERE `SID` = '$SID' LIMIT 1	");
		}

		# Sets the handler

		public function set_session_handler()	{

			$this->_active=TRUE;
			
			ini_set('session.use_cookies', '1');
			ini_set('session.use_only_cookies', '1');

			session_set_save_handler (   	
				array(&$this, "session_open"),
				array(&$this, "session_close"),
				array(&$this, "session_read"),
				array(&$this, "session_write"),
				array(&$this, "session_destroy"),
				array(&$this, "session_gc_fake")
			);
		}
	}
?>
