<?php
	/**
	 *	eBag
	 *
	 *	Open source company administration manager
	 *
	 *	@package	eBag
	 *	@author		Eper Kalman
	 *	@copyright	Copyright (c) 2012-2015, Eper Kalman
	 *	@license	GNU Affero General Public License Version 3 (http://www.gnu.org/licenses/agpl-3.0.html)
	 *	@since		Version 6.0.0
	 */

	/**
	 * eBag upload files
	 */

	load_f('mk_dir');
	
	if (! function_exists('file_extension')) load_f('file_extension');
	
	class UPLOAD	{

		var $INPUTNAME	=	'upload';

		var $REJECTED	=	array('exe', 'com');

		# note: maximum upload size depends of php settings to

		var $MAX_MB	=	15;

		var $MAXFILES	=	1000;

		function __construct($TYPE, $UPLOADDIR)	{

			$this->TYPE		=	$TYPE;

			$this->UPLOADDIR	=	$UPLOADDIR;

			global $__SQL, $__CONF;

			$this->SQL		=	&$__SQL;

			$this->ClamAV		=	$__CONF->get('virusscan','clamavpath');
		}

		# check file count in upload dir and create new one if is full

		public function prepare_dir($directoryID)	{

			$DIRECTORY	=	"$this->UPLOADDIR/upl$directoryID";

			if (! is_dir($DIRECTORY)) {

				if (! mk_dir($DIRECTORY)) return FALSE;

				return $directoryID;
			}

			# store files by 1000 in one folder

			$count = 0;

			if ($dh = opendir($DIRECTORY))	{

				while (($file = readdir($dh)) != FALSE )	{

					$count++;

					if ($count > ($this->MAXFILES + 1))	{

						$FULL = TRUE;
						break;
					}
				}
			}

			if ($dh) closedir($dh);

			if (isset($FULL))	{

				while (TRUE)	{

					$directoryID++;

					$DIRECTORY	=	"$this->UPLOADDIR/upl$directoryID";

					if (! is_dir($DIRECTORY)) break;
				}

				if (! mk_dir($DIRECTORY)) return FALSE;
			}

			return $directoryID;
		}

		# check filesize and rejected extensions
	
		private function accepted()	{

			if ((isset($this->REJECTED)) && (count($this->REJECTED)))	{

				$ext = file_extension($_FILES[$this->INPUTNAME]['name']);

				foreach ($this->REJECTED as $EXTENSION)	{

					if (strtolower($ext) == strtolower($EXTENSION))	{

						$errtxt = 'notallowedextension';

						break;
					}
				}
			}

			if (isset($errtxt)) return $errtxt;

			if ($_FILES[$this->INPUTNAME]['size'] > ($this->MAX_MB*1024*1024)) return 'tobig2upload';

			return TRUE;
		}

		# scan file with clamAV

		function clamavscan($file)	{

			$code=-1;

			@exec($this->ClamAV.' --stdout '.escapeshellarg($file), $out, $code);

			# no virus found

			if ($code == 0) return FALSE;

			# virus found, delete the file
			
			if ($code == 1) 	{

				@unlink($file);

				return TRUE;
			}

			# clamAV error

			if ($code == 2)	{

				echo "Warning: file $file was not scanned.<br>Report :<br>";

				if ((isset($out)) && (count($out)))	{

					foreach ($out as $line)	echo "$line<br>";
				}
			}

			return FALSE;
		}

		# upload

		public function store()	{

			# get last directory ID

			$this->SQL->query("	SELECT MAX(directoryID) AS `directoryID`
						FROM ebag.uploads ");

			$RESULT = $this->SQL->RESULT->fetch_row();

			if (! $directoryID = $RESULT[0]) $directoryID = 1;

			# verify the directory is not full

			if (! $directoryID = $this->prepare_dir($directoryID)) return 'cannotmakeuploaddir';

			$DIRECTORY = "$this->UPLOADDIR/upl$directoryID";

			# set time limit

			$timelimit = ini_get('max_execution_time');

			set_time_limit(0);

			# start upload

			if (! is_uploaded_file($_FILES[$this->INPUTNAME]['tmp_name']))	  {

				if (($_FILES[$this->INPUTNAME]['error']) && ($_FILES[$this->INPUTNAME]['error']!=4)) return 'uploaderror';
			}

			# check filesize and extension

			$RESULT = $this->accepted();

			if ($RESULT !== TRUE) return $RESULT;

			# optionally scan with clamAV 

			if ($this->ClamAV)	{

				if ($this->clamavscan($_FILES[$this->INPUTNAME]['tmp_name'])) return 'virusfounddeleted';
			}

			# note: types p=>personal, c=>company

			if ($this->TYPE == 'c')		{

				$owner	=	$this->SQL->n_escape(md5('company'));

			}	else	  {

				$owner	=	$this->SQL->n_escape($_SESSION['username']);
			}

			$filename	=	$this->SQL->n_escape($_FILES[$this->INPUTNAME]['name']);

			$filesize		=	$_FILES[$this->INPUTNAME]['size'];

			$mimetype	=	$this->SQL->n_escape($_FILES[$this->INPUTNAME]['type']);

			$date_of_creation =	date("Y-m-d");

			# start transaction because maybe the move will fail

			$this->SQL->transaction();

			$this->SQL->query("	INSERT INTO ebag.uploads
						VALUES
						(NULL, '$this->TYPE', $owner, '$directoryID', $filename, '$filesize', $mimetype , '$date_of_creation')	");

			$fileID	=	$this->SQL->lastid();

			$REALNAME	=	"upl$fileID.upl";

			if (! @move_uploaded_file($_FILES[$this->INPUTNAME]['tmp_name'], "$DIRECTORY/$REALNAME"))	{

				$this->SQL->rollback();

				return 'uploaderror';
			}

			$this->SQL->commit();

			# restore time limit

			set_time_limit($timelimit);

			return TRUE;
		}
	}
?>

