<?php
	/**
	 *	eBag
	 *
	 *	Open source company administration manager
	 *
	 *	@package	eBag
	 *	@author		Eper Kalman
	 *	@copyright	Copyright (c) 2012-2015, Eper Kalman
	 *	@license	GNU Affero General Public License Version 3 (http://www.gnu.org/licenses/agpl-3.0.html)
	 *	@since		Version 6.0.0
	 */

	/**
	 * eBag years administration (close, open, create_new);
 	 */

	class YEARSADMIN	{

		function __construct()	{

			global $__SQL, $__SESS;
			
			$this->SQL		=	&$__SQL;

			$this->SESS		=	&$__SESS;
			
			$this->FILEMARK		=	BASEDIR.'tmp/maintenance.conf';
			
			$this->BACKUPMARK 	=	FALSE;
		}
		
		# generate backupmark filename 
		
		public function backupdir($dir=FALSE)	{

			if (! $dir) return;
			
			$dir = safe_dir($dir);

			if (is_dir($dir)) $this->BACKUPMARK = "$dir/backup_started.tmp";
		}

		# determine existents of year in base

		private function year_exist($YEAR)	{

			$this->SQL->query("	SELECT SCHEMA_NAME 
						FROM INFORMATION_SCHEMA.SCHEMATA 
						WHERE SCHEMA_NAME = 'ebagdata$YEAR'	");

			if ($this->SQL->num_rows()) return TRUE;

			return FALSE;
		}

		private function loggeds_count()	{

			$COUNT = 0;

			$this->SQL->query("	SELECT `personID`
						FROM ebag.users
						WHERE `deleted` IS NOT NULL	");

			while ($RESULT = $this->SQL->RESULT->fetch_row())	{

				if ($this->SESS->is_logged($this->SQL->r_escape($RESULT[0]))) $COUNT++;
			}

			return $COUNT;
		}

		private function is_unfinished_jobs($YEAR)	{

			$this->SQL->query("	(	
							SELECT `personID` 
							FROM ebagdata$YEAR.orders_data 
							WHERE `date_of_delivery` IS NULL AND `deleted` IS NOT NULL LIMIT 1
						)
						UNION ALL
						(
							SELECT `personID` 
							FROM ebagdata$YEAR.deliverys_header 
							WHERE `billID` IS NULL AND `deleted` IS NOT NULL LIMIT 1
						)
						UNION ALL
						(
							SELECT `personID` 
							FROM ebagdata$YEAR.services_header 
							WHERE `billID` IS NULL AND `deleted` IS NOT NULL LIMIT 1
						)	");

			if ($this->SQL->num_rows()) return TRUE;

			return FALSE;
		}

		private function roll_back()	{

			$this->SQL->rollback();

			# delete maintenance mark

			@unlink($this->FILEMARK);
		}

		public function open($YEAR)	{
		
			# check for other process
			
			if (is_file($this->FILEMARK)) return 'error->maintenanceinprogress';
			
			# do not work with years if the backup is in progress
			
			if ($this->BACKUPMARK && is_file($this->BACKUPMARK)) return 'error->backupinprogress';

			# check that is no more logged users

			if ($this->loggeds_count() != 1) return 'error->moreofoneloggeds';

			# sets file mark for maintenance which will
			# redirect the users temporary from login

			@file_put_contents($this->FILEMARK, 'eBag_maintenance_in_progress');

			$this->SQL->transaction();

			# verify the year exists

			if (! $this->year_exist($YEAR))	{

				$this->roll_back();

				return 'error->unexistentyear';
			}

			# verify the year is not closed already

			$this->SQL->query("	SELECT `year` 
						FROM ebag.years
						WHERE `year`='$YEAR' AND `closed` IS NULL LIMIT 1	");

			if ($this->SQL->num_rows())	{

				$this->roll_back();

				return 'error->alreadyopenedyear';
			}

			# drop archiv stock tables

			$TABLES	= array('finished_goods_stock', 'feedstock', 'operating_suplies', 'services', 'feedstock_requests_list');

			foreach ($TABLES as $TABLE)	{

				$this->SQL->query("DROP TABLE IF EXISTS ebagdata$YEAR.$TABLE	");
			}

			# update years

			$this->SQL->query("	UPDATE ebag.years
						SET `closed`=NULL
						WHERE `year`='$YEAR' LIMIT 1	");

			$this->SQL->commit();

			# if this is current year open it at now

			if (($_SESSION['year'] == $YEAR) && (isset($_SESSION['year_is_closed']))) unset ($_SESSION['year_is_closed']);

			# delete maintenance mark

			@unlink($this->FILEMARK);

			return 'openedyear';
		}

		public function close($YEAR)	{
		
			# check for other process
			
			if (is_file($this->FILEMARK)) return 'error->maintenanceinprogress';
			
			# do not work with years if the backup is in progress
			
			if ($this->BACKUPMARK && is_file($this->BACKUPMARK)) return 'error->backupinprogress';

			# check that is no more logged users

			if ($this->loggeds_count() != 1) return 'error->moreofoneloggeds';

			# sets file mark for maintenance which will
			# redirect the users temporary from login

			@file_put_contents($this->FILEMARK, 'eBag_maintenance_in_progress');

			$this->SQL->transaction();

			# verify the year exists

			if (! $this->year_exist($YEAR))	{

				$this->roll_back();

				return 'error->unexistentyear';
			}

			# verify the year is not closed already

			$this->SQL->query("	SELECT `year`
						FROM ebag.years
						WHERE `year`='$YEAR' AND `closed` IS NOT NULL LIMIT 1");

			if ($this->SQL->num_rows())	{

				$this->roll_back();

				return 'error->alreadyclosedyear';
			}

			if ($this->is_unfinished_jobs($YEAR))	{

				$this->roll_back();

				return 'error->unfinishedjobs';
			}

			# copy the stock bases into defined base to archive

			$TABLES	= array('finished_goods_stock', 'feedstock', 'operating_suplies', 'services', 'feedstock_requests_list');

			foreach ($TABLES as $TABLE)	{

				$this->SQL->query("	CREATE TABLE ebagdata$YEAR.$TABLE 
							LIKE ebagstock.$TABLE	");

				$this->SQL->query("	INSERT INTO ebagdata$YEAR.$TABLE
							SELECT * FROM ebagstock.$TABLE	");
			}

			# update years

			$this->SQL->query("	UPDATE ebag.years
						SET `closed`='1' 
						WHERE `year`='$YEAR' LIMIT 1	");

			$this->SQL->commit();

			# if this is current year close it at now

			if ($_SESSION['year'] == $YEAR) $_SESSION['year_is_closed']='TRUE';

			# delete maintenance mark

			@unlink($this->FILEMARK);

			return 'closedyear';
		}

		public function create_new($YEAR)	{
		
			# check for other process
			
			if (is_file($this->FILEMARK)) return 'error->maintenanceinprogress';
			
			# do not work with years if the backup is in progress
			
			if ($this->BACKUPMARK && is_file($this->BACKUPMARK)) return 'error->backupinprogress';

			# check that is no more logged users

			if ($this->loggeds_count() != 1) return 'error->moreofoneloggeds';

			# sets file mark for maintenance which will
			# redirect the users temporary from login

			@file_put_contents($this->FILEMARK, 'eBag_maintenance_in_progress');

			$this->SQL->transaction();

			# verify the year exists

			if ($this->year_exist($YEAR))	{

				$this->roll_back();

				return 'error->existyear2create';
			}

			# note:	select the last yeat from base to copy tables 
			# 	(maybe the current year is not the last)

			$this->SQL->query("	SELECT MAX(`year`) AS `year` 
						FROM ebag.years	");

			if ($this->SQL->num_rows())	{

				$RESULT = $this->SQL->RESULT->fetch_row();

				$this->LASTYEAR = $RESULT[0];

			}	else	  {

				$this->roll_back();

				return 'error->unknownerror';
			}

			$TO_COPY = array(	'addressbook', 'claims', 'customers_payments', 'employees', 'salarys_personal_data', 'employees_items_bonuses', 'employees_phases_bonuses' );

			$this->SQL->query("	CREATE DATABASE `ebagdata$YEAR` DEFAULT CHARACTER SET utf8 COLLATE utf8_bin");

			$this->SQL->query("	SHOW TABLES FROM `ebagdata$this->LASTYEAR`	");

			if (! $this->SQL->num_rows())	{

				$this->roll_back();

				return 'error->unknownerror';
			}

			$this->SQL->IT1_RESULT = $this->SQL->RESULT;

			while ($IT1_RESULT = $this->SQL->IT1_RESULT->fetch_row())	{

				$TABLE	= $IT1_RESULT[0];

				# create new table

				$this->SQL->query("	CREATE TABLE ebagdata$YEAR.$TABLE 
							LIKE ebagdata$this->LASTYEAR.$TABLE	");

				if (in_array($TABLE, $TO_COPY))		{

					# copy data into from the last year

					$this->SQL->query("	INSERT INTO ebagdata$YEAR.$TABLE 
								SELECT * 
								FROM ebagdata$this->LASTYEAR.$TABLE	");
				}
			}
			
			# employees_bonuses have archived rows delete it to speed up
			
			$this->SQL->query("	DELETE FROM ebagdata$YEAR.employees_items_bonuses WHERE `deleted` IS NULL ");
			
			$this->SQL->query("	DELETE FROM ebagdata$YEAR.employees_phases_bonuses WHERE `deleted` IS NULL ");

			# update years

			$this->SQL->query("	INSERT INTO ebag.years 
						VALUES 
						('$YEAR',NULL)	");

			$this->SQL->commit();

			# delete maintenance mark

			@unlink($this->FILEMARK);

			return 'newyearcreated';
		}
	}

?>
