<?php
	/**
	 *	eBag
	 *
	 *	Open source company administration manager
	 *
	 *	@package	eBag
	 *	@author		Eper Kalman
	 *	@copyright	Copyright (c) 2012-2015, Eper Kalman
	 *	@license	GNU Affero General Public License Version 3 (http://www.gnu.org/licenses/agpl-3.0.html)
	 *	@since		Version 5.0.0
	 */

	/**
	 * eBag reads order data for create new production order
	 */
	 
	if (! defined('eBag'))	{
	
	      include('../views/logout.php'); 
	      
	      exit();
	}
	 
	load_f('date2sql');
	
	load_f('datecompare');
	
	load_f('is_real_positive');

	load_f('is_int_positive');

	class PRODUCTIONORDER2STOCKDEFAULT	{

		function __construct()	{

			global $__SQL, $__CONF, $__LANG;

			$this->SQL		=	&$__SQL;

			$this->FORMAT		=	$__CONF->get('date', 'dateformat');

			$this->UNIT		=	new UNITloader($__CONF->get('cache', 'use_apc_cache'));	
			
			$this->STOCKTXT		=	$__LANG->get('customerstock');
			
			$this->BASE		=	$_SESSION['BASE'];
			
			$this->STOCKBASE	=	'ebagstock';

		}

		private function lock($stocktype)	{

			$LOCKS	= array();

			foreach ($this->YEARS as $YEAR)	$LOCKS[] = " ebagdata$YEAR.production_orders_header WRITE, ebagdata$YEAR.production_orders_data WRITE ";

			if (count($LOCKS) >= 1) $LOCKS = implode(',', $LOCKS);
			
			if ($stocktype == 1)	{
			
			      $STOCKLOCK = "$this->STOCKBASE.finished_goods_stock WRITE";
			      
			}	else	  {
			
			      $STOCKLOCK = "$this->STOCKBASE.operating_suplies WRITE";
			}

			$this->SQL->query("LOCK TABLES	$STOCKLOCK,
							$this->BASE.orders_header WRITE,
							$this->BASE.orders_data WRITE, ".$LOCKS);
		}

		private function unlock()	{
		
			$this->SQL->query('UNLOCK TABLES');
		}

		private function readyears()	{
		
			$this->YEARS = array();

			$this->SQL->query("	SELECT `year`
						FROM ebag.years ORDER BY `year` DESC	");

			while ($RESULT = $this->SQL->RESULT->fetch_row()) $this->YEARS[] = $RESULT[0];
		}

		public function read($DATA)	{
		
			# check data
			
			$date_of_creation = date2sql($DATA['date_of_creation'], $this->FORMAT);
		
			$date_of_deadline = date2sql($DATA['date_of_deadline'], $this->FORMAT);
		
			if ((! $date_of_creation) or (! $date_of_deadline)) return 'dateerror';
		
			if (datecompare($date_of_creation, '>', $date_of_deadline)) return 'dateerror';

			$this->readyears();

			$this->lock($DATA['stocktype']);

			# read stock data check existens
			# note : table keys from view select
			
			$STOCKTABLES = array( 1 => 'finished_goods_stock',
					      2 => 'operating_suplies');
					      
			$STOCKNAMES  = array( 1 => 'articalname',
					      2 => 'supliename');
					      
			$stocktable = $STOCKTABLES[$DATA['stocktype']];
			
			$stockname  = $STOCKNAMES[$DATA['stocktype']];
			
			$code = $this->SQL->n_escape($DATA['articalcode']);
			
			# note: if is artical only products can be produced
			
			if ($DATA['stocktype']==1)	{
			
				$ONLYPRODUCT = ' AND `origin`=\'1\' ';
				
			}	else	{
			
				$ONLYPRODUCT = '';
			}

			$this->SQL->query("	SELECT `$stockname`, `unit`
						FROM $this->STOCKBASE.$stocktable
						WHERE `code`=$code $ONLYPRODUCT LIMIT 1	");

			if (! $this->SQL->num_rows())	{

				$this->unlock();

				return 'unexistentitem';
			}
			
			$RESULT = $this->SQL->RESULT->fetch_assoc();
			
			$DATA['articalname'] = $RESULT[$stockname];
			
			# now check quantity by unit
			
			if ($this->UNIT->is_int_unit($RESULT['unit']))	{

				if (! is_int_positive($DATA['articalquantity'])) $error = 'usererror';
					
			}	else	 {

				if (! is_real_positive($DATA['articalquantity'])) $error = 'usererror';
			}
			
			if (isset($error))	{
			
				$this->unlock();
				
				return $error;
			}

			# form orders header data
			
			$DATA['orderID'] = '-';
			
			$DATA['contactname'] = $this->STOCKTXT;

			# if is produced before read the last technology

			foreach ($this->YEARS as $YEAR)		{

				$this->SQL->query("	SELECT MAX(`personID`) 
							FROM ebagdata$YEAR.production_orders_header 
							WHERE `code`=$code AND `deleted` IS NOT NULL	");

				$RESULT	=	$this->SQL->RESULT->fetch_row();

				if ($RESULT[0])		{

					$productionorderID	=	$RESULT[0];
					$productionYEAR		=	$YEAR;
					
					break;
				}
			}

			$DATA['technology']	=	'';
			$DATA['drawingID']	=	'';

			if (isset($productionorderID))		{

				# read technology

				$this->SQL->query("	SELECT `technology`, `drawingID` 
							FROM ebagdata$productionYEAR.production_orders_header
							WHERE `personID`='$productionorderID' AND `deleted` IS NOT NULL LIMIT 1	");

				$RESULT	=	$this->SQL->RESULT->fetch_row();

				$DATA['technology']	=	$RESULT[0];
				$DATA['drawingID']	=	$RESULT[1];

				# also read the feedstocks & articles
	
				$this->SQL->query("	SELECT `code`, `name`, `unit`, `quantity`, `type`  
							FROM ebagdata$productionYEAR.production_orders_data
							WHERE `productionorderID`='$productionorderID' AND `deleted` IS NOT NULL ORDER BY `personID` ");

				if ($this->SQL->num_rows())	{
				
					$DATA['code'] = $DATA['name'] = $DATA['quantity'] = $DATA['unit'] = $DATA['type'] = array();

					while ($RESULT = $this->SQL->RESULT->fetch_assoc())	{

						$DATA['code'][]		=	$RESULT['code'];
						$DATA['name'][]		=	$RESULT['name'];
						$DATA['quantity'][]	=	$RESULT['quantity'];
						$DATA['unit'][]		=	$RESULT['unit'];
						$DATA['type'][]		=	$RESULT['type'];
					}
				}
			}

			$this->unlock();

			return $DATA;
		}
	}

?>
