<?php
	/**
	 *	eBag
	 *
	 *	Open source company administration manager
	 *
	 *	@package	eBag
	 *	@author		Eper Kalman
	 *	@copyright	Copyright (c) 2012-2015, Eper Kalman
	 *	@license	GNU Affero General Public License Version 3 (http://www.gnu.org/licenses/agpl-3.0.html)
	 *	@since		Version 6.0.0
	 */

	/**
	 * eBag production phases settings (add new, change, set to default or delete)
	 */
	 
	if (! defined('eBag'))	{
	
	      include('../views/logout.php'); 
	      
	      exit();
	}

	# note: the default production phase cannot be changed 
	#	because the production always begins in feedstock
	
	load_f('is_longer');

	class PRODUCTIONPHASES	{

		function __construct()	{

			global $__SQL;

			$this->SQL	=	&$__SQL;

			$this->readyears();
		}

		private function lock()	{

			$TOLOCK	= '';

			foreach ($this->YEARS as $YEAR)	$TOLOCK .= ", ebagdata$YEAR.production_orders_phases WRITE , 
								      ebagdata$YEAR.employees_phases_bonuses WRITE ";

			$this->SQL->query('LOCK TABLES ebag.years WRITE, ebag.production_phases WRITE'.$TOLOCK);
		}

		private function unlock()	{

			$this->SQL->query('UNLOCK TABLES');
		}

		private function readyears()	{
		
			$this->YEARS = array();

			$this->SQL->query("	SELECT `year` 
						FROM ebag.years 
						ORDER BY `year`	");

			while ($RESULT = $this->SQL->RESULT->fetch_row()) $this->YEARS[] = $RESULT[0];

		}

		private function is_in_use($phaseID)	{

			$FOUND = FALSE;

			foreach ($this->YEARS as $YEAR)	{

				$this->SQL->query("	(
							  SELECT `personID` 
							  FROM ebagdata$YEAR.production_orders_phases
							  WHERE `production_phase`='$phaseID' LIMIT 1	
							)
							UNION ALL
							(
							  SELECT `personID`
							  FROM ebagdata$YEAR.employees_phases_bonuses
							  WHERE `phaseID`='$phaseID' LIMIT 1
							)
						  ");

				if ($this->SQL->num_rows())	{

					$FOUND = TRUE;

					break;
				}
			}

			return $FOUND;
		}

		public function change($phaseID, $NAME)	{

			if (is_empty($NAME) or is_longer($NAME, 40)) return 'usererror';

			$this->lock();

			if ($this->is_in_use($phaseID))	{

				$this->unlock();

				return 'cannotchangeusedphase';
			}
		
			$NAME = $this->SQL->n_escape($NAME);

			# check the new name not exists once

			$this->SQL->query("	SELECT `personID`
						FROM ebag.production_phases
						WHERE `phase`=$NAME LIMIT 1	");

			if ($this->SQL->num_rows())	{

				$this->unlock();

				return 'productionphaseexists';
			}

			$this->SQL->query("	UPDATE ebag.production_phases
						SET `phase`=$NAME
						WHERE `personID`='$phaseID' LIMIT 1	");

			$this->unlock();
	
			return TRUE;
		}

		public function delete($phaseID)	{

			$this->lock();

			if ($this->is_in_use($phaseID))	{

				$this->unlock();

				return 'cannotdeleteusedphase';
			}

			# check that is not default

			$this->SQL->query("	SELECT `personID` 
						FROM ebag.production_phases
						WHERE `personID`='$phaseID' AND `default` IS NOT NULL LIMIT 1	");

			if ($this->SQL->num_rows())	{

				$this->unlock();

				return 'cannotdeletedefaultphase';
			}

			# check that is not the last

			$this->SQL->query("	SELECT `phase`
						FROM ebag.production_phases	");

			if ($this->SQL->num_rows()==1)	{

				$this->unlock();

				return 'cannotdeletelastphase';
			}

			$this->SQL->query("	DELETE
						FROM ebag.production_phases
						WHERE `personID`='$phaseID' LIMIT 1	");

			$this->unlock();

			return TRUE;
		}

		public function addnew($NAME)	{

			if (is_empty($NAME) or is_longer($NAME, 40)) return 'usererror';

			# note: no need to lock all tables

			$this->SQL->query('LOCK TABLES ebag.production_phases WRITE');

			$NAME	=	$this->SQL->n_escape($NAME);

			# check the new name not exists once

			$this->SQL->query("	SELECT `personID`
						FROM ebag.production_phases
						WHERE `phase`=$NAME LIMIT 1	");

			if ($this->SQL->num_rows())	{

				$this->unlock();

				return 'productionphaseexists';
			}

			$this->SQL->query("	INSERT INTO ebag.production_phases
						VALUES
						(NULL, $NAME, NULL)	");

			$this->unlock();
	
			return TRUE;
		}

	}

?>
