<?php
	/**
	 *	eBag
	 *
	 *	Open source company administration manager
	 *
	 *	@package	eBag
	 *	@author		Eper Kalman
	 *	@copyright	Copyright (c) 2012-2015, Eper Kalman
	 *	@license	GNU Affero General Public License Version 3 (http://www.gnu.org/licenses/agpl-3.0.html)
	 *	@since		Version 3.0.2
	 */

	/**
	 * eBag service list finisher write
	 */
	 
	if (! defined('eBag'))	{
	
	      include('../views/logout.php'); 
	      
	      exit();
	}

	load_f('date2sql');

	class SERVICELISTFINISHERWRITE	{

		# message stores information about action opened/closed

		var $MESSAGE	=	'';

		function __construct()	{

			global $__SQL, $__CONF;

			$this->SQL		=	&$__SQL;
			
			$this->FORMAT		=	$__CONF->get('date', 'dateformat');
			
			$this->BASE		=	$_SESSION['BASE'];
			
			$this->STOCKBASE	=	'ebagstock';
		}

		private function lock()	{

			$this->SQL->query("LOCK TABLES	$this->BASE.services_header WRITE, 
							$this->BASE.services_data WRITE, 
							$this->BASE.addressbook WRITE, 
							$this->BASE.employees WRITE,
							$this->STOCKBASE.services WRITE, 
							$this->STOCKBASE.feedstock WRITE, 
							$this->STOCKBASE.feedstock_requests_list WRITE, 
							$this->BASE.feedstock_requests WRITE	");
		}

		private function unlock()	{

			$this->SQL->query("UNLOCK TABLES");
		}

		private function prepare($DATA)	{

			if (! is_empty($DATA['date_of_finishing']))	{

				if (is_empty($DATA['finisher'])) return 'usererror';

				$DATA['date_of_finishing']	=	date2sql($DATA['date_of_finishing'], $this->FORMAT);
				$DATA['date_of_creation']	=	date2sql($DATA['date_of_creation'], $this->FORMAT);

				if ((! $DATA['date_of_finishing']) or (! $DATA['date_of_creation'])) return 'dateerror';

				# see the date of finishing is older than creation

				list ($F_YY, $F_MM, $F_DD)	=	explode('-', $DATA['date_of_finishing']);
				list ($C_YY, $C_MM, $C_DD)	=	explode('-', $DATA['date_of_creation']);

				if (mktime(0, 0, 0, $F_MM, $F_DD, $F_YY) <  mktime(0, 0, 0, $C_MM, $C_DD, $C_YY)) return 'dateerror';

			}

			# note: escape now without quotes because later will be checked

			foreach ($DATA as $key=>$val) $DATA[$key] = $this->SQL->s_escape($val);

			return $DATA;
		}

		public function finish($DATA)	{

			$ID	=	$this->SQL->escape($DATA['id']);

			unset($DATA['id']);
			
			$DATA = $this->prepare($DATA);

			if (! is_array($DATA)) return $DATA;

			$this->SQL->transaction();

			$this->lock();

			# check the service list exists yet

			$this->SQL->query("	SELECT `billID`, `date_of_finishing` 
						FROM $this->BASE.services_header 
						WHERE `personID`='$ID' AND `deleted` IS NOT NULL LIMIT 1	");

			if ($this->SQL->num_rows())	{

				$RESULT	= $this->SQL->RESULT->fetch_assoc();

				if ($RESULT['billID'])	$errtxt = 'billedrevers';

				if ($RESULT['date_of_finishing']) {$ALREADY_WAS_CLOSED = TRUE;} else {$ALREADY_WAS_CLOSED = FALSE;}

			}	else	{

				$errtxt	= 'unexistentrevers';
			}

			if (isset($errtxt))	{

				$this->SQL->rollback();

				$this->unlock();

				return $errtxt;
			}

			# check the defined finisher exists yet

			if ($DATA['date_of_finishing'])		{

				list($TYPE, $FINISHERID)	=	explode('_', $DATA['finisher']);

				if ($TYPE == 'employee')	{

					$COMMAND = "	SELECT `employeename` 
							FROM $this->BASE.employees 
							WHERE `personID`='$FINISHERID' AND `deleted` IS NOT NULL LIMIT 1	";

				}	else	   {

					$COMMAND = "	SELECT `contactname` 
							FROM $this->BASE.addressbook 
							WHERE `personID`='$FINISHERID' AND `deleted` IS NOT NULL LIMIT 1	";
				}

				$this->SQL->query($COMMAND);

				if (! $this->SQL->num_rows())	{

					$this->SQL->rollback();

					$this->unlock();

					return 'finisherdeleted';
				}
			}


			if ($DATA['date_of_finishing'])	  	{

				# do close

				if (! $ALREADY_WAS_CLOSED)	{

					# check the revers needs feedstock and request list is created or not

					$this->SQL->query("	SELECT `code` 	FROM $this->BASE.services_data WHERE `reversID`='$ID' AND `deleted` IS NOT NULL 
								AND `code` IN (
								SELECT services.code	FROM $this->STOCKBASE.services INNER JOIN $this->STOCKBASE.feedstock_requests_list
								   			ON services.personID=feedstock_requests_list.serviceID 
											WHERE services.deleted IS NOT NULL AND feedstock_requests_list.deleted IS NOT NULL)");

					if ($this->SQL->num_rows())	{

						$this->SQL->query("	SELECT `personID` 
									FROM $this->BASE.feedstock_requests 
									WHERE `reversID`='$ID' AND `deleted` IS NOT NULL LIMIT 1	");

						if (! $this->SQL->num_rows())	{

							$this->SQL->rollback();

							$this->unlock();

							return 'nofeedstockrequestcreated';
						}

						# update the stock

						$this->SQL->query("	SELECT `code`, `quantity` 
									FROM $this->BASE.feedstock_requests 
									WHERE `reversID`='$ID' AND `deleted` IS NOT NULL	");

						$this->SQL->IT1_RESULT	= $this->SQL->RESULT;

						while ($RESULT = $this->SQL->IT1_RESULT->fetch_assoc())	{

							$CODE		=	$this->SQL->r_escape($RESULT['code']);
							$QUANTITY	=	$RESULT['quantity'];

							$this->SQL->query("	UPDATE $this->STOCKBASE.feedstock 
										SET quantity_in_production=quantity_in_production-$QUANTITY 
										WHERE `code`='$CODE' AND `deleted` IS NOT NULL LIMIT 1	");
						}

						unset($this->SQL->IT1_RESULT);
					}
				}

				# close it

				extract($DATA);

				$this->SQL->query("	UPDATE $this->BASE.services_header 	
							SET `finished`='1', `date_of_finishing`='$date_of_finishing', `finisher`='$finisher'
	  						WHERE `personID`='$ID' LIMIT 1		");

				$this->MESSAGE	=	'servicelistclosed';

			}	else	  {

				# do open

				if ($ALREADY_WAS_CLOSED)	{

					# update the stock

					$this->SQL->query("	SELECT `code`, `quantity` 
								FROM $this->BASE.feedstock_requests 
								WHERE `reversID`='$ID' AND `deleted` IS NOT NULL	");

					$this->SQL->IT1_RESULT	= $this->SQL->RESULT;

					while ($RESULT = $this->SQL->IT1_RESULT->fetch_assoc())	{

						$CODE		=	$this->SQL->r_escape($RESULT['code']);
						$QUANTITY	=	$RESULT['quantity'];

						$this->SQL->query("	UPDATE $this->STOCKBASE.feedstock 
									SET quantity_in_production=quantity_in_production+$QUANTITY 
									WHERE `code`='$CODE' AND `deleted` IS NOT NULL LIMIT 1	");
					}

					unset($this->SQL->IT1_RESULT);
				}

				$this->SQL->query("	UPDATE $this->BASE.services_header 
							SET `finished`='0', `date_of_finishing`=NULL, `finisher`=NULL 
							WHERE `personID`='$ID' LIMIT 1	");

				$this->MESSAGE	=	'servicelistopened';

			}

			$this->SQL->commit();

			$this->unlock();

			return TRUE;
		}
	}

?>
