<?php
	/**
	 *	eBag
	 *
	 *	Open source company administration manager
	 *
	 *	@package	eBag
	 *	@author		Eper Kalman
	 *	@copyright	Copyright (c) 2012-2015, Eper Kalman
	 *	@license	GNU Affero General Public License Version 3 (http://www.gnu.org/licenses/agpl-3.0.html)
	 *	@since		Version 6.0.0
	 */

	/**
	 * eBag service list write
	 */
	 
	if (! defined('eBag'))	{
	
	      include('../views/logout.php'); 
	      
	      exit();
	}
	
	load_f('date2sql');

	load_f('is_real_positive');

	load_f('is_int_positive');

	load_f('datecompare');

	class SERVICELISTWRITE	{

		function __construct()	{

			global $__SQL, $__CONF;

			$this->SQL		=	&$__SQL;

			$this->CONF		=	&$__CONF;

			$this->UNIT		=	new UNITloader($this->CONF->get('cache', 'use_apc_cache'));

			$this->BASE		=	$_SESSION['BASE'];

			$this->STOCKBASE	=	'ebagstock';
		}

		public function getid()	{

			if (isset($this->reversID)) {return $this->reversID;} else {return FALSE;}
		}

		private function lock()	{

			$this->SQL->query("	LOCK TABLES  	$this->BASE.addressbook WRITE, 
								$this->BASE.services_header WRITE, 
								$this->BASE.services_data WRITE, 
								$this->STOCKBASE.services WRITE, 
								ebag.companydata WRITE	");
		}

		private function unlock()	{

			$this->SQL->query('UNLOCK TABLES');
		}

		private function prepare($DATA)	{

			# note: the prices will be rereaded form base as check and prevent change in meantime

			$CODES		=	$DATA['code'];
			$NAMES		=	$DATA['name'];
			$QUANTITYS	=	$DATA['quantity'];
			$UNITS		=	$DATA['unit'];

			$LENGTH	=	count($CODES);

			if (! $LENGTH) return 'usererror';

			$RETURN	=	array();

			for ($i=0; $i<$LENGTH; $i++)	{

				$code		=	$CODES[$i];
				$name		=	$NAMES[$i];
				$quantity	=	$QUANTITYS[$i];
				$unit		=	$UNITS[$i];

				# empty rows

				if ((is_empty($code)) && (is_empty($name)) && (is_empty($quantity)) && (is_empty($unit))) continue;
				
				# Initialize vars
				    
				if (! isset($RETURN['code'])) 		$RETURN['code'] = array();
				if (! isset($RETURN['name'])) 		$RETURN['name'] = array();
				if (! isset($RETURN['quantity'])) 	$RETURN['quantity'] = array();
				if (! isset($RETURN['unit'])) 		$RETURN['unit'] = array();
				if (! isset($RETURN['price'])) 		$RETURN['price'] = array();
				if (! isset($RETURN['tax'])) 		$RETURN['tax'] = array();

				# user input

				if ((is_empty($code)) or (is_empty($name))) $RETURN = 'usererror';

				if ($this->UNIT->is_int_unit($unit))	{

					if (! is_int_positive($quantity)) $RETURN = 'usererror';
						
				}	else	 {

					if (! is_real_positive($quantity)) $RETURN = 'usererror';
				}				

				if ($RETURN == 'usererror') break;

				# service exists

				$code	=	$RETURN['code'][]	=	$this->SQL->n_escape($code);

				$this->SQL->query("	SELECT `price`, `tax` 
							FROM $this->STOCKBASE.services 
							WHERE `code`=$code AND `deleted` IS NOT NULL LIMIT 1	");

				if (! $this->SQL->num_rows()) {$RETURN = 'servicenotexist'; break;}

				$RESULT	=	$this->SQL->RESULT->fetch_row();
				
				$RETURN['name'][]	=	$this->SQL->n_escape($name);
				$RETURN['quantity'][]	=	$this->SQL->n_escape($quantity);
				$RETURN['unit'][]	=	$this->SQL->n_escape($unit);
				$RETURN['price'][]	=	$this->SQL->nr_escape($RESULT[0]);
				$RETURN['tax'][]	=	$this->SQL->nr_escape($RESULT[1]);
			}
	
			if ((is_array($RETURN)) && (! count($RETURN))) $RETURN = 'usererror';

			return $RETURN;
		}

		public function write($DATA)	{

			# prepare & check header data

			if 	(((is_empty($DATA['contactID'])) && (is_empty($DATA['contactname']))) or 

				(is_empty($DATA['date_of_creation'])) or

				(is_empty($DATA['date_of_deadline'])) or

				(is_empty($DATA['payment_form']))) return 'usererror';


			$contactID		=	$this->SQL->n_escape($DATA['contactID']);

			$contactname		=	$this->SQL->n_escape($DATA['contactname']);

			$date_of_creation	=	date2sql($DATA['date_of_creation'], $this->CONF->get('date', 'dateformat'));

			$date_of_deadline	=	date2sql($DATA['date_of_deadline'], $this->CONF->get('date', 'dateformat'));

			$payment_form		=	$this->SQL->n_escape($DATA['payment_form']);

			if ((! $date_of_creation) or (! $date_of_deadline)) return 'dateerror';

			# see the date of deadline is older than creation

			if (datecompare($date_of_creation, '>', $date_of_deadline)) return 'dateerror';

			$comment		=	$this->SQL->n_escape($DATA['comment']);
			$annotation		=	$this->SQL->n_escape($DATA['annotation']);

			if (! isset($DATA['use_session_discount']))	$DATA['use_session_discount'] = '';

			$use_session_discount	=	$this->SQL->n_escape($DATA['use_session_discount']);

			$session_discount	=	$this->SQL->n_escape($DATA['session_discount']);

			if (! isset($DATA['use_service_discount']))	$DATA['use_service_discount'] = '';

			$use_service_discount	=	$this->SQL->n_escape($DATA['use_service_discount']);

			# note: service discount will reread from base


			# start transaction

			$this->SQL->transaction();
		
			$this->lock();

			# read & check contact

			if ($contactID == 'NULL')	{

				$this->SQL->query("	SELECT `personID` 
							FROM $this->BASE.addressbook 
							WHERE `contactname`=$contactname LIMIT 1	");

				if ($this->SQL->num_rows())	{

					$RESULT	=	$this->SQL->RESULT->fetch_row();

					$contactID	=	$RESULT[0];
				}	
			}

			if ($contactname == 'NULL')	{

				$this->SQL->query("	SELECT `personID` 
							FROM $this->BASE.addressbook 
							WHERE `personID`=$contactID LIMIT 1	");

				if ($this->SQL->num_rows())	{

					$RESULT	=	$this->SQL->RESULT->fetch_row();

					$contactname	=	$this->SQL->nr_escape($RESULT[0]);
				}	
			}

			$this->SQL->query("	SELECT `personID` 
						FROM $this->BASE.addressbook 
						WHERE `personID`=$contactID AND `contactname`=$contactname LIMIT 1	");

			if (! $this->SQL->num_rows())	{

				$this->SQL->rollback();

				$this->unlock();

				return 'contactnotmatch';
			}

			# prepare service data
			
			$DATA = $this->prepare($DATA);

			if ((! isset($DATA['code'])) or (! $LENGTH = count($DATA['code']))) $DATA = 'usererror';

			if (! is_array($DATA)) 	{

				$this->SQL->rollback();

				$this->unlock();

				return $DATA;
			}

			# set & read essentials

			$exchange_rate	=	$this->SQL->n_escape($this->CONF->get('payments', 'exchange_rate'));

			$currency	=	$this->SQL->n_escape($this->CONF->get('payments', 'currency'));
			
			$valuta		=	$this->SQL->n_escape($this->CONF->get('payments', 'valuta'));

			$companyID	=	FALSE;

			$this->SQL->query("	SELECT MAX(`personID`) AS `personID` 
						FROM ebag.companydata	");

			if ($this->SQL->num_rows())	{

				$RESULT		=	$this->SQL->RESULT->fetch_row();
				$companyID	=	$RESULT[0];
			}

			if (($exchange_rate == 'NULL') or ($currency == 'NULL') or ($valuta == 'NULL') or (! $companyID))	{

				$this->SQL->rollback();

				$this->unlock();

				if (! $companyID) {return 'nocompanyID';} else {return 'fatalerror';}
			}

			$companyID	=	"'$companyID'";

			# read actual contact address

			$this->SQL->query("	SELECT	`country`, `city`, `zip`, `street`, `delivery_country`, `delivery_city`, `delivery_street`, `delivery_zip`, 
							`service_discount`, `account1`, `bank1`, `account2`, `bank2`, `contacttype`, `seller`, `delivery_day_of_week` 
						FROM $this->BASE.addressbook
						WHERE `personID`=$contactID AND `deleted` IS NOT NULL LIMIT 1");


			# note: must to be row because after lock was checked before

			$RESULT	=	$this->SQL->RESULT->fetch_assoc();

			foreach ($RESULT as $key=>$val)	$RESULT[$key]=$this->SQL->nr_escape($val);

			extract($RESULT); unset($RESULT);

			# insert service list header

			$this->SQL->query("INSERT INTO $this->BASE.services_header VALUES (	NULL, $companyID, $contactID, $contactname, $country, $city, $zip, $street, 
												$delivery_country, $delivery_city, $delivery_street, $delivery_zip, 
												$service_discount, $account1, $bank1, $account2, $bank2, $contacttype, 
												$seller, '$date_of_creation', '$date_of_deadline', $payment_form, $delivery_day_of_week, 
												'0', NULL, NULL, '0', NULL, $comment, $annotation, $currency, $valuta, $exchange_rate,
												$session_discount, $use_session_discount, $use_service_discount, NULL, '0')");

			$this->reversID = $this->SQL->lastid();
			
			if (! $this->reversID)	{
			
				$this->SQL->rollback();

				$this->unlock();

				return 'unknownerror';
			}

			# insert service list data
			
			$LENGTH = count($DATA['code']);

			for ($i=0; $i<$LENGTH; $i++)	{

				$code		=	$DATA['code'][$i];
				$servicename	=	$DATA['name'][$i];
				$quantity	=	$DATA['quantity'][$i];
				$unit		=	$DATA['unit'][$i];
				$price		=	$DATA['price'][$i];
				$tax		=	$DATA['tax'][$i];
				
				$this->SQL->query("	INSERT INTO $this->BASE.services_data 
							VALUES  
							(NULL, $this->reversID, $code, $servicename, $quantity, $unit, $price, $tax, '0', '0')	");
			}
			
			$this->SQL->commit();

			$this->unlock();

			return TRUE;
		}
	}

?>
