<?php
	/**
	 *	eBag
	 *
	 *	Open source company administration manager
	 *
	 *	@package	eBag
	 *	@author		Eper Kalman
	 *	@copyright	Copyright (c) 2012-2015, Eper Kalman
	 *	@license	GNU Affero General Public License Version 3 (http://www.gnu.org/licenses/agpl-3.0.html)
	 *	@since		Version 6.0.0
	 */

	/**
	 * eBag write order to stock after creating or updating production order
	 */
	 
	if (! defined('eBag'))	{
	
	      include('../views/logout.php'); 
	      
	      exit();
	}

	load_f('is_real_positive');

	load_f('is_int_positive');

	class STOCKORDER2PRODUCTIONWRITE	{

		function __construct()	{

			global $__SQL, $__CONF;

			$this->SQL		=	&$__SQL;

			$this->CONF		=	&$__CONF;

			$this->UNIT		=	new UNITloader($this->CONF->get('cache', 'use_apc_cache'));

			$this->BASE		=	$_SESSION['BASE'];

			$this->STOCKBASE	=	'ebagstock';
		}

		private function lock()	{

			$this->SQL->query("	LOCK TABLES 	$this->STOCKBASE.feedstock WRITE, 
								$this->STOCKBASE.finished_goods_stock WRITE, 
								$this->BASE.addressbook WRITE, 
								$this->BASE.production_orders_header WRITE, 
								$this->BASE.orders_to_stock_header WRITE,
								$this->BASE.orders_to_stock_data WRITE, 
								ebag.companydata WRITE	");
		}

		private function unlock()	{

			$this->SQL->query('UNLOCK TABLES');
		}

		private function prepare($DATA)		{

			$CODES		=	$DATA['code'];
			$QUANTITYS	=	$DATA['order'];
			$UNITS		=	$DATA['unit'];
			$TYPES		=	$DATA['type'];

			$LENGTH	=	count($CODES);

			if (! $LENGTH) return 'usererror';

			$RETURN	=	array();

			for ($i=0; $i<$LENGTH; $i++)	{

				$code		=	$CODES[$i];
				$quantity	=	$QUANTITYS[$i];
				$unit		=	$UNITS[$i];
				$type		=	$TYPES[$i];
				
				# Step over empty lines and zero quantities

				if ((is_empty($code) && is_empty($quantity) && is_empty($unit)) or (is_numeric($quantity) && (! $quantity))) continue;
				
				# Initialize vars
				
				if (! isset($RETURN['code'])) 		$RETURN['code'] = array();
				if (! isset($RETURN['quantity'])) 	$RETURN['quantity'] = array();
				if (! isset($RETURN['type'])) 		$RETURN['type'] = array();
				if (! isset($RETURN['supplierID']))	$RETURN['supplierID'] = array();

				# user input

				if ($this->UNIT->is_int_unit($unit))	{

					if (! is_int_positive($quantity)) $RETURN = 'usererror';
						
				}	else	 {

					if (! is_real_positive($quantity)) $RETURN = 'usererror';
				}				

				if ($RETURN == 'usererror') break;

				# feedstock or artical exists

				$code	=	$RETURN['code'][]	=	$this->SQL->n_escape($code);
				
				if ($type == 'a') {$stock = 'finished_goods_stock'; $return_str = 'articalnotexist';} else {$stock = 'feedstock'; $return_str = 'feedstocknotexist';}

				$this->SQL->query("	SELECT `supplierID`
							FROM $this->STOCKBASE.$stock
							WHERE `code`=$code AND `deleted` IS NOT NULL LIMIT 1");

				if (! $this->SQL->num_rows())	{

					$RETURN = $return_str;

					break;
				}

				$RESULT	=	$this->SQL->RESULT->fetch_row();

				$RETURN['quantity'][]	=	$quantity;
				$RETURN['type'][]	=	$this->SQL->n_escape($type);
				$RETURN['supplierID'][]	=	$this->SQL->nr_escape($RESULT[0]);
			}
	
			if ((is_array($RETURN)) && (! count($RETURN))) $RETURN = 'usererror';

			return $RETURN;
		}

		public function write($DATA)	{

			$productionorderID	=	$DATA['productionorderID'];

			# set & read essentials

			$exchange_rate	=	$this->SQL->n_escape($this->CONF->get('payments', 'exchange_rate'));

			$currency	=	$this->SQL->n_escape($this->CONF->get('payments', 'currency'));

			# start transaction

			$this->SQL->transaction();
		
			$this->lock();

			$DATA = $this->prepare($DATA);

			if (! is_array($DATA)) 	{

				$this->SQL->rollback();

				$this->unlock();

				return $DATA;
			}

			$companyID	=	FALSE;

			$this->SQL->query("SELECT MAX(`personID`) AS `personID` FROM ebag.companydata");

			if ($this->SQL->num_rows())	{

				$RESULT		=	$this->SQL->RESULT->fetch_row();
				$companyID	=	"'".$RESULT[0]."'";
			}

			if (($exchange_rate == 'NULL') or ($currency == 'NULL') or (! $companyID))	{

				$this->SQL->rollback();

				$this->unlock();

				if (! $companyID) {return 'nocompanyID';} else {return 'fatalerror';}
			}

			# read production order for orderID connected with & check exists yet

			$this->SQL->query("	SELECT `orderID`
						FROM $this->BASE.production_orders_header 
						WHERE `personID`='$productionorderID' AND `deleted` IS NOT NULL LIMIT 1		");

			if (! $this->SQL->num_rows())	{

				$this->SQL->rollback();

				$this->unlock();

				return 'unexistentproductionorders->personIDlong->'.$productionorderID;
			}

			$RESULT		=	$this->SQL->RESULT->fetch_row();

			$orderID	=	$RESULT[0];

			$COUNT = count($DATA['code']);

			for ($i=0; $i<$COUNT; $i++)	{

				$code		=	$DATA['code'][$i];
				$type		=	$DATA['type'][$i];
				$quantity	=	$DATA['quantity'][$i];
				$supplierID	=	$DATA['supplierID'][$i];

				# important to force recheck connected with orders to every artical

				if (isset($connected_with_orders)) unset($connected_with_orders);

				# check for opened order created to this supplier

				$this->SQL->query("	SELECT `personID` FROM $this->BASE.orders_to_stock_header
							WHERE `supplierID`=$supplierID
							AND `ordered`='0' AND `deleted` IS NOT NULL LIMIT 1	");

				if ($this->SQL->num_rows())	{

					# get the exists (opened) order2stockID

					$RESULT		=	$this->SQL->RESULT->fetch_row();

					$order2stockID	=	$RESULT[0];

				}	else	  {

					# open a new order to stock to this supplier

					$this->SQL->query("	SELECT	`contactname`, `country`, `city`, `zip`, `street`, `account1`,
							 		`bank1`, `account2`, `bank2` FROM $this->BASE.addressbook
								WHERE 	`personID`=$supplierID AND `deleted` IS NOT NULL LIMIT 1	");

					if (! $this->SQL->num_rows())	{

						$errtxt	=	'err_supplierdeleted->personID->'.str_replace("'", '', $suplierID);

						break;
					}

					$RESULT	=	$this->SQL->RESULT->fetch_assoc();

					foreach ($RESULT as $key=>$val)	$RESULT[$key] = $this->SQL->nr_escape($val);

					extract($RESULT);
					
					# orderID = 0 means production is to stock
					
					if ($orderID)	{

						$connected_with_orders	=	$this->SQL->n_escape("-$orderID-");
					
					}	else	  {
					
						$connected_with_orders	=	'NULL';
					}

					$this->SQL->query("	INSERT INTO $this->BASE.orders_to_stock_header VALUES
								(NULL, $companyID, $supplierID, $contactname, $country, $city, $zip, $street, $account1, $bank1, 
								$account2, $bank2, NULL, $currency, $exchange_rate, $connected_with_orders, '0', '0', NULL, NULL, '0')	");

					$order2stockID	=	$this->SQL->lastid();
				}

				# if is old order update connected with orders column if the production is not for stock

				if (! isset($connected_with_orders) && $orderID)	{

					$this->SQL->query("	SELECT `connected_with_orders` FROM $this->BASE.orders_to_stock_header
								WHERE `personID`='$order2stockID' AND `connected_with_orders` NOT REGEXP '-$orderID-' LIMIT 1	");

					$connected_with_orders	=	$this->SQL->n_escape("-$orderID-");

					if ($this->SQL->num_rows())	{

						$RESULT	=	$this->SQL->RESULT->fetch_row();

						# note:	connected_with_orders format -1-4-5-

						if ($RESULT[0]) $connected_with_orders = $this->SQL->n_escape($RESULT[0]."$orderID-");
					}

					$this->SQL->query("	UPDATE $this->BASE.orders_to_stock_header
								SET `connected_with_orders`=$connected_with_orders
						  		WHERE `personID`='$order2stockID' LIMIT 1	");
				}

				# check for artical writed before
				
				if ($type == "'f'") {$stock = 'feedstock'; $name = 'feedstockname';} else {$stock = 'finished_goods_stock'; $name = 'articalname';}
			
				$this->SQL->query("	SELECT `personID` FROM $this->BASE.orders_to_stock_data
				  			WHERE `code`=$code AND `type`=$type AND `order2stockID`='$order2stockID' AND `deleted` IS NOT NULL LIMIT 1	");

				if ($this->SQL->num_rows())	{

					# update

					$this->SQL->query("	UPDATE $this->BASE.orders_to_stock_data SET quantity=quantity+$quantity
								WHERE `code`=$code AND `type`=$type AND `order2stockID`='$order2stockID' AND `deleted` IS NOT NULL LIMIT 1 ");
				}	else	  {

					# insert new one
					
					$this->SQL->query("	SELECT `$name` as `itemname`, `unit`, `purchase_price` FROM $this->STOCKBASE.$stock
								WHERE	`code`=$code AND `deleted` IS NOT NULL LIMIT 1	");

					# note: must be row, checked after lock before

					$RESULT	=	$this->SQL->RESULT->fetch_assoc();
					
					# note: very important to change null price to 0 because of backward compatibility
					#	the column cannot be null and readed price can be null in previous versions
					
					if (! $RESULT['purchase_price']) $RESULT['purchase_price'] = '0';

					foreach ($RESULT as $key=>$val)	$RESULT[$key] = $this->SQL->nr_escape($val);

					extract($RESULT);

					$this->SQL->query("	INSERT INTO $this->BASE.orders_to_stock_data 
								VALUES
								(NULL, '$order2stockID', $code, $itemname, '$quantity', $unit, $purchase_price, NULL, NULL, NULL, '0', $type, '0')	");
				}

				# update the stock

				$this->SQL->query("	UPDATE $this->STOCKBASE.$stock
							SET quantity_on_opened_order_to_stock=quantity_on_opened_order_to_stock+$quantity
							WHERE `code`=$code AND `deleted` IS NOT NULL LIMIT 1	");
			}
			
			# for any case check for empty orders
			
			$this->SQL->query("	SELECT `personID` FROM $this->BASE.orders_to_stock_data
						WHERE `order2stockID`='$order2stockID' AND `deleted` IS NOT NULL LIMIT 1	");

			if (! $this->SQL->num_rows()) $errtxt = 'emptyorder';

			if (isset($errtxt))	{

				$this->SQL->rollback();

				$this->unlock();

				return $errtxt;
			}

			$this->SQL->commit();

			$this->unlock();

			return TRUE;
		}

	}
?>
